local ADDON, Package = ...
local Enum = Package.Enum
local Color = Package.Color
local ColorPalette = Package.ColorPalette
local Snapshot = Package.Snapshot
local Message = Package.Message

local agent = CreateFrame("Frame")
local registry = {}

local function migrate()
  if MedusaTradeStorage and MedusaTradeStorage.version and MedusaTradeStorage.version < 3 then
    MedusaTradeStorage = {}
  end
end

local function load_storage()
  local schema = {
    version = 3,
    whisper = true,
    history = {},
    transaction = nil,
  }

  MedusaTradeStorage = Enum.merge(MedusaTradeStorage or {}, schema, false)
end

local function boot_up_messages()
  local addon_name = Color.format(ColorPalette.JORDY_BLUE, ADDON)
  local addon_version = Color.format(ColorPalette.JORDY_BLUE, "v" .. GetAddOnMetadata(ADDON, "Version"))
  local addon_slashes = Color.format(ColorPalette.SPRAY, _G["SLASH_" .. ADDON .. "1"] .. " " .. _G["SLASH_" .. ADDON .. "2"])

  DEFAULT_CHAT_FRAME:AddMessage(addon_name .. " " .. addon_version .. " " .. addon_slashes )
end

local Trade = {}

function Trade:accept()
  local state = MedusaTradeStorage

  table.insert(state.history, 1, Enum.clone(state.transaction))

  if #state.history > 512 then
    table.remove(state.history)
  end

  Message:accept(state.transaction)
end

function Trade:cancel()
  local state = MedusaTradeStorage

  Message:cancel(state.transaction)
end

function Trade:update()
  local state = MedusaTradeStorage

  state.transaction = Snapshot:new()
end

function registry.ADDON_LOADED(self, ...)
  if ADDON == ... then
    migrate()
    load_storage()
    boot_up_messages()
  end
end

function registry.UI_INFO_MESSAGE(self, error_type, message)
  if error_type == LE_GAME_ERR_TRADE_COMPLETE then
    local state = MedusaTradeStorage

    if state.whisper then
      Trade:accept()
    end

    local listbox = _G[ADDON .. "HistoryWindowTabPane1ListBox"]

    if listbox then
      listbox:ForceUpdate()
    end
  end

  if error_type == LE_GAME_ERR_TRADE_CANCELLED then
    local state = MedusaTradeStorage

    if state.whisper then
      Trade:cancel()
    end
  end
end

function registry.TRADE_PLAYER_ITEM_CHANGED(self)
  Trade:update()
end

function registry.TRADE_TARGET_ITEM_CHANGED(self)
  Trade:update()
end

function registry.TRADE_ACCEPT_UPDATE(self)
  Trade:update()
end

function registry.TRADE_UPDATE(self)
  Trade:update()
end

function registry.TRADE_SHOW(self)
  Trade:update()
end

agent:RegisterEvent("ADDON_LOADED")
agent:RegisterEvent("UI_INFO_MESSAGE")
agent:RegisterEvent("TRADE_PLAYER_ITEM_CHANGED")
agent:RegisterEvent("TRADE_TARGET_ITEM_CHANGED")
agent:RegisterEvent("TRADE_ACCEPT_UPDATE")
agent:RegisterEvent("TRADE_UPDATE")
agent:RegisterEvent("TRADE_SHOW")
agent:SetScript("OnEvent", function (self, event, ...) registry[event](self, ...) end)

_G["SLASH_" .. ADDON .. "1"] = "/" .. ADDON
_G["SLASH_" .. ADDON .. "2"] = "/거래내역"

SlashCmdList[ADDON] = function ()
  if not MedusaTradeHistoryWindow then
    Package.history()
  end

  MedusaTradeHistoryWindow:SetShown(not MedusaTradeHistoryWindow:IsShown())
end
