local ADDON, Package = ...
local Enum = {}

function Enum.clone(t)
  local stash = {}

  for k, v in pairs(t) do
    if type(v) == "table" then
      stash[k] = Enum.clone(v)
    else
      stash[k] = v
    end
  end

  return stash
end

function Enum.omit(t, key, depth)
  local stash = {}

  for k, v in pairs(t) do
    if k ~= key then
      if type(v) == "table" and depth and depth > 0 then
        stash[k] = Enum.omit(v, key, depth - 1)
      else
        stash[k] = v
      end
    end
  end

  return stash
end

function Enum.rename(t, from, to)
  local stash = {}

  for k, v in pairs(t) do
    local i = k == from and to or k

    if type(v) == "table" then
      stash[i] = Enum.rename(v, from, to)
    else
      stash[i] = v
    end
  end

  return stash
end

function Enum.merge(to, from, override)
  local stash = Enum.clone(to)

  for k, v in pairs(from) do
    if type(v) == "table" and type(stash[k]) == "table" then
      stash[k] = Enum.merge(stash[k], v, override)
    else
      if stash[k] == nil or override ~= false then
        stash[k] = v
      end
    end
  end

  return stash
end

function Enum.concat(left, right)
  local left = Enum.clone(left)
  local right = Enum.clone(right)
  local stash = {}

  for _, entry in pairs(left) do
    table.insert(stash, entry)
  end

  for _, entry in pairs(right) do
    table.insert(stash, entry)
  end

  return stash
end

function Enum.is_member(t, value)
  for _, entry in pairs(t) do
    if entry == value then
      return true
    end
  end

  return false
end

Package.Enum = Enum
