local ADDON, Package = ...

local Snapshot = {}

function Snapshot:get_map()
  local id = C_Map.GetBestMapForUnit("player")
  local x = 0
  local y = 0
  local pvp = GetZonePVPInfo()
  local zone = GetZoneText() or UNKNOWN
  local subzone = GetSubZoneText()


  if id then
    position = C_Map.GetPlayerMapPosition(id, "player")

    if position then
      x, y = position:GetXY()
    end
  end

  return id, x, y, pvp, zone, subzone
end

function Snapshot:get_character(unit)
  local sex = UnitSex(unit)
  local _, race = UnitRace(unit)
  local name = UnitNameUnmodified(unit)
  local class = UnitClassBase(unit)
  local level = UnitLevel(unit)
  local guild, _, _ = GetGuildInfo(unit)

  return sex == 2 and "male" or "female", string.lower(race), name, string.lower(class), level, guild
end

function Snapshot:get_money(unit)
  local unit = string.gsub(unit, "^[a-z]", string.upper)
  local GetTradeMoney = _G["Get" .. unit .. "TradeMoney"]
  local money = GetTradeMoney()

  return money
end

function Snapshot:get_items(unit)
  local unit = string.gsub(unit, "^[a-z]", string.upper)
  local GetTradeItemInfo = _G["GetTrade" .. unit .. "ItemInfo"]
  local GetTradeItemLink = _G["GetTrade" .. unit .. "ItemLink"]
  local items = {}

  for at = 1, MAX_TRADE_ITEMS do
    local name, icon, quantity, quality, enchantment, unknown = GetTradeItemInfo(at)
    local enchantment = unknown or enchantment
    local link = GetTradeItemLink(at)
    local id = string.match(link or "", "|Hitem:(%d+).-|h")

    if id and name and quantity then
      items[at] = {
        id = tonumber(id),
        name = name,
        icon = icon,
        quality = quality,
        quantity = quantity,
        enchantment = type(enchantment) == "string" and enchantment or nil
      }
    else
      items[at] = false
    end
  end

  return items
end

function Snapshot:new()
  local map_id, map_x, map_y, map_pvp, map_zone, map_subzone = self:get_map()

  local player_sex, player_race, player_name, player_class, player_level, player_guild = self:get_character("player")
  local player_money = self:get_money("player")
  local player_items = self:get_items("player")

  local target_sex, target_race, target_name, target_class, target_level, target_guild = self:get_character("npc")
  local target_money = self:get_money("target")
  local target_items = self:get_items("target")

  local timestamp = GetServerTime()

  return {
    map_id = map_id,
    map_x = map_x,
    map_y = map_y,
    map_pvp = map_pvp,
    map_zone = map_zone,
    map_subzone = map_subzone,

    player_sex = player_sex,
    player_race = player_race,
    player_name = player_name,
    player_class = player_class,
    player_level = player_level,
    player_guild = player_guild,
    player_money = player_money,
    player_items = player_items,

    target_sex = target_sex,
    target_race = target_race,
    target_name = target_name,
    target_class = target_class,
    target_level = target_level,
    target_guild = target_guild,
    target_money = target_money,
    target_items = target_items,

    timestamp = timestamp,
  }
end

Package.Snapshot = Snapshot
